import { TypeOptions, SerializeType } from 'src/common';
import * as applicationJson from 'src/types/application/json';
import * as applicationXml from 'src/types/application/xml';
import * as applicationXWwwFormUrlencoded from 'src/types/application/x-www-form-urlencoded';

const TYPES: Record<string, SerializeType> = {
  'application/json': applicationJson,
  'text/json': applicationJson,
  'application/xml': applicationXml,
  'text/xml': applicationXml,
  'application/x-www-form-urlencoded': applicationXWwwFormUrlencoded,
};

export const AVAILABLE_TYPES: (keyof typeof TYPES)[] = Object.keys(TYPES);

export const serialize = <T, O extends TypeOptions = TypeOptions>(
  data: T,
  options: O,
): string => {
  const { type: optionsType } = options;
  const { [optionsType]: serializeType } = TYPES;
  if (!serializeType) {
    throw new Error(`Unsupported type: ${optionsType}`);
  }
  return serializeType.serialize(data, options);
};

export const deserialize = <T, O extends TypeOptions = TypeOptions>(
  serialized: string,
  options: O,
): T => {
  const { type: optionsType } = options;
  const { [optionsType]: serializeType } = TYPES;
  if (!serializeType) {
    throw new Error(`Unsupported type: ${optionsType}`);
  }
  return serializeType.deserialize<T>(serialized, options);
};
